<?php
require "config.php";

// 添加 CodeMirror 资源函数
function includeCodeMirrorViewer($mode = 'text/plain') {
    echo <<<HTML
    <!-- 引入 CodeMirror Material Dark 主题 -->
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/codemirror.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/theme/material-darker.min.css">
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/codemirror.min.js"></script>
    <!-- 引入常用语言模式 -->
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/htmlmixed/htmlmixed.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/xml/xml.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/javascript/javascript.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/css/css.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/clike/clike.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/php/php.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/shell/shell.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.65.2/mode/markdown/markdown.min.js"></script>
    <style>
        .CodeMirror {
            border: 1px solid #444;
            height: auto;
            font-size: 14px;
            line-height: 1.5;
            margin-bottom: 10px;
            background: #212121;
        }
        .CodeMirror-scroll {
            min-height: 400px;
        }
        .cm-s-material-darker .CodeMirror-gutters {
            background: #212121 !important;
            border-right: 1px solid #333;
        }
        #codeViewer {
            display: none;
        }
    </style>
    <script>
    document.addEventListener('DOMContentLoaded', function() {
        var editor = CodeMirror.fromTextArea(document.getElementById('codeViewer'), {
            lineNumbers: true,
            mode: '$mode',
            theme: 'material-darker',
            readOnly: true,
            indentUnit: 4,
            lineWrapping: true
        });
    });
    </script>
HTML;
}

// 根据文件扩展名获取 CodeMirror 模式
function getEditorMode($filename) {
    $extension = pathinfo($filename, PATHINFO_EXTENSION);
    $modeMap = [
        'php' => 'application/x-httpd-php',
        'js' => 'javascript',
        'html' => 'htmlmixed',
        'css' => 'css',
        'c' => 'clike',
        'cpp' => 'clike',
        'java' => 'clike',
        'sh' => 'shell',
        'xml' => 'xml',
        'json' => 'javascript',
        'txt' => 'text/plain',
        'md' => 'markdown',
        // 添加更多文件类型映射
    ];
    return $modeMap[strtolower($extension)] ?? 'text/plain';
}

if (!isset($_GET['path'])) {
    header("Location: ./404.php");
    exit;
} elseif (($path = trim($_GET['path'])) == "") {
    header("Location: ./404.php");
    exit;
} elseif (!is_file($path)) {
    header("Location: ./404.php");
    exit;
} elseif (!is_readable($path)) {
    xhtml_head("查看文件");
    echo "<div class=\"like\">\n";
    echo "<a href=\"./index.php?path=" . urlencode(dirname($path)) . "\"]>返回目录</a>错误提示\n";
    echo "</div>\n";
    echo "<div class=\"love\">\n";
    echo "此文件您没有读取权限！";
    echo "</div>\n";
    xhtml_footer();
} else {
    // 获取文件扩展名对应的编辑器模式
    $editorMode = getEditorMode($path);
    
    xhtml_head("查看文件");
    // 包含 CodeMirror 资源
    includeCodeMirrorViewer($editorMode);
    
    echo "<div class=\"like\">\n";
    echo "<a href=\"./index.php?path=" . urlencode(dirname($path)) . "\"]>返回目录</a>选择编码\n";
    echo "</div>\n";
    echo "<div class=\"love\">";
    echo "<form action=\"\" method=\"GET\">\n";
    echo "<select name=\"charset\">\n";
    if (!function_exists("mb_convert_encoding")) {
        echo "<option>缺少模块</option>\n";
    } else {
        $sencode = mb_list_encodings();
        usort($sencode, "___sortcmp");
        foreach ($sencode as $encode) {
            if ($encode == "pass") {
                continue;
            }
            if (function_exists("mb_encoding_aliases")) {
                $alias = mb_encoding_aliases($encode);
                echo "\n<optgroup label=\"$encode\">\n";
                echo "<option value=\"$encode\">$encode</option>\n";
                if (is_array($alias)) if (count($alias) >= 1) {
                    usort($alias, "___sortcmp");
                    foreach ($alias as $encodealias) {
                        if ($encodealias == $encode) {
                            continue;
                        }
                        echo "<option value=\"$encode\">$encodealias</option>\n";
                    }
                }
                echo "</optgroup>\n";
            } else {
                echo "<option value=\"$encode\">$encode</option>\n";
            }
        }
    }
    echo "</select>\n";
    echo "<input type=\"hidden\" name=\"path\" value=\"$path\" />";
    echo "<input type=\"submit\" value=\"设置编码\" />\n";
    echo "</form>\n";
    echo "</div>\n";
    echo "<div class=\"like\">爱特文件内容浏览</div>\n";
    if (filesize($path) > (2 * 1024 * 1024)) {
        echo "<div class=\"love\">\n";
        echo "文件过大，请下载后查看！\n";
        echo "</div>\n";
    } else {
        echo "<div class=\"love\">\n";
        if (!($data = file_get_contents($path))) {
            echo "读取文件时发生了错误！\n";
        } else {
            if (!isset($_GET['charset'])) {
                $content = ___codepre(___convert($data, "UTF-8"));
            } elseif (($charset = trim($_GET['charset'])) == "") {
                $content = ___codepre(___convert($data, "UTF-8"));
            } else {
                $content = ___codepre(___convert($data, "UTF-8", $charset));
            }
            // 使用 CodeMirror 显示代码
            echo "<textarea id=\"codeViewer\">$content</textarea>\n";
        }
        echo "</div>";
    }
    xhtml_footer();
}
?>